#!/usr/bin/env python

import smbus


class AlarmDevice(object):
    _CMD_LEADIN = 0xfe
    _CMD_NOOP = 0x00
    _CMD_ALLOFF = 0x01
    _CMD_SETRGB = 0x02
    _CMD_SETX = 0x04
    _CMD_XPULSE = 0x05
    _CMD_BALANCE = 0x08

    def __init__(self, bus):
        self._i2caddr = 0x44
        self._i2cbus = smbus.SMBus(bus) if type(bus) is int else None
        self._send_command(self._CMD_BALANCE, 255, 170, 90)

    def off(self):
        self._send_command(self._CMD_ALLOFF)

    def set_rgb(self, r, g, b):
        """r, g, b in [0,255]"""
        self._send_command(self._CMD_SETRGB, r, g, b)

    def set_beep(self, on):
        self._send_command(self._CMD_SETX, 1 if on else 0)

    def pulse_beep(self, t):
        self._send_command(self._CMD_XPULSE, t)

    def _send_command(self, cmd, param1=0, param2=0, param3=0):
        if self._i2cbus is None:
            return
        packet = [cmd & 0xff, param1 & 0xff, param2 & 0xff, param3 & 0xff]
        packet.append(reduce(int.__xor__, packet, self._CMD_LEADIN))
        retries = 3
        while retries > 0:
            try:
                self._i2cbus.write_block_data(
                    self._i2caddr,
                    self._CMD_LEADIN,
                    packet)
                break
            except IOError:
                retries -= 1
